function [ret,phan]=Aqual2_findSNRSphere(phan,theory,verbose)
if ~exist('verbose','var')
    verbose=0;
end
ret=0;

filename=phan.filename;

% Assume we know nothing about where the phantom is within the FOV
propList={ 'Area', 'Centroid', 'BoundingBox','SubarrayIdx','PixelList','PixelIdxList'};
requiredVolume=0.8*sum([theory.Spheres(:).Volume])/phan.voxelVolume;
nFoundVoxels=0;

currentThresh=OtsuThreshold(phan.img);
% most of the time  Otsu's method will work for this
% make sure we found enough voxels, if not fall back lowering threshold
binaryImg=phan.img>currentThresh;
nFoundVoxels=sum(binaryImg(:));

if nFoundVoxels<requiredVolume
    if (verbose>0)
        disp('searching for SNR sphere -- Otsu threshold was apparently too high');
        disp(' reducing threshold');
    end

    %  do histogram binning to find the threshold
    nBins=phan.hdr.dime.glmax-phan.hdr.dime.glmin;
    if(nBins>1000)
        nBins=1000;
    end
    [cont,binEdges]=hist(phan.img(:),nBins);
    csum=cumsum(cont(end:-1:1));
    clear cont;
    binEdges=binEdges(end:-1:1);
    index=find(csum>requiredVolume);
    currentThresh=binEdges(index(1));
    clear binEdges;
    clear csum;
    binaryImg=phan.img>currentThresh;
    nFoundVoxels=sum(binaryImg(:));

    if(nFoundVoxels<(requiredVolume*0.90))
        ret=1;
        estring=sprintf('cannot find enough voxels when hunting for SNR sphere ');
        Aqual2_writeFailureLog(phan.outputFilePrefix,estring);
        return;
    end
end

labelImg=bwlabeln(binaryImg);
props=regionprops(labelImg,propList);
clear labelImg;
targetVolume= theory.Spheres(theory.SNRSphereIndex).Volume/phan.voxelVolume;
requiredVolume= 0.5 *  targetVolume;

[v,sortOrder]=sort([props.Area]);
if(v(end) <requiredVolume)
    ret=1;
    estring=sprintf('cannot find single cluster of voxels large enough  when hunting for SNR sphere ');
    Aqual2_writeFailureLog(phan.outputFilePrefix,estring);
    return;
else
    % we have likely  found the sphere we want now clean up the sphere
    % Essentially we restrict the FOV to only around the sphere; this
    % possibly makes us less susceptible to large scale inhomogeneity
    % effects
    region=props(sortOrder(end));
    grow=2;  % get atleast a two voxel margin
    regionImage=[];
    while(size(regionImage(:))<targetVolume*8/pi)
        % dilate out a couple of voxels and get image in the neighborhood
        [subIndexVec,OffsetVec]=GetSubImageIndex(region.SubarrayIdx,grow,size(phan.img));
        regionImage=phan.img(subIndexVec{1},subIndexVec{2},subIndexVec{3});
        grow=grow+1;
    end

    [region.thresh,region.effectiveness]=OtsuThreshold(regionImage);
    % now we put a window on the volume found of 0 to +20% of nominal
    % not symmetric since we may have included parts of neighboring
    % navigator spheres
    while(sum(regionImage(:)>region.thresh) < 1.0*targetVolume)
        region.thresh=region.thresh*0.98;
    end
    while(sum(regionImage(:)>region.thresh) > 1.2*targetVolume)
        region.thresh=region.thresh*1.02;
    end
    % sanity check: fail if region.thresh hit either the 99% of the
    % intensity max of the image or 1% of the intensity min
    if(region.thresh>(0.99*max(regionImage(:))))
        ret=1;
        estring=sprintf('cannot find a suitable threshold for SNR sphere (too high) ');
        Aqual2_writeFailureLog(phan.outputFilePrefix,estring);
        return;
    elseif(region.thresh<(0.01*max(regionImage(:))))
        ret= 1;
        estring=sprintf('cannot find a suitable threshold for SNR sphere (too low) ');
        Aqual2_writeFailureLog(phan.outputFilePrefix,estring);
        return;
    end
    % should have a region.thresh that's not impossibly preposterous
    regionBinaryImg=(regionImage>region.thresh);
    regionLabelImg=bwlabeln(regionBinaryImg);
    % assume the center of the region contains the object we desire
    center=round(size(regionLabelImg)/2);
    center = sub2ind ( size(regionLabelImg), center(1), center(2), center(3) );
    val=regionLabelImg(center);
    regionLabelImg=bwlabeln(regionLabelImg==val);
    if   ((max(regionLabelImg(:))>1) || (min(regionLabelImg(:))<0))
        ret= 1;
        estring=sprintf('bad classification in Aqual2_findSNRSphere -- this should never happen ');
        Aqual2_writeFailureLog(phan.outputFilePrefix,estring);
        return;
    end
    % while we are here and have the grey scale image with mask, then let's
    % go ahead and caluclate the SNR
    mask=(regionLabelImg==1);
    regionImage=double(regionImage);
    [phan.SNR,phan.S,phan.N]=CalculateSNRFromSphere ( regionImage,mask);
    % now we need to put the labels into the master phan.masterLabelImg;
    sz=size(regionLabelImg);
    % you can do this the sloppy matlab way  below and it'll be 100x slower than
    %  the c-like array looping implementation
    %     tic
    %      where=find(regionLabelImg);
    %       delta=OffsetVec-[1,1,1];
    %
    %     for ii=1:length(where)
    %         [x,y,z]=ind2sub(sz,where(ii));
    %         x=x+delta(1);
    %         y=y+delta(2);
    %         z=z+delta(3);
    %        phan.masterLabelImg(x,y,z)=...
    %             theory.SNRSphereIndex;
    %     end
    % toc


    % tic
    for ii=1:sz(1)
        for jj=1:sz(2)
            for kk=1:sz(3)
                phan.masterLabelImg(ii+OffsetVec(1)-1,jj+OffsetVec(2)-1,kk+OffsetVec(3)-1)=...
                    theory.SNRSphereIndex*regionLabelImg(ii,jj,kk);
            end
        end
    end
    %toc

    phan=findObjectPositions(phan,theory.SNRSphereIndex);

    return;

end

