function Transform = CoordinateSystems ( Headers, Type )
% Transform = CoordinateSystems ( Header )
% Computes the transformation between matrix coordinates (row, col,
% slice) and RAS scanner coordinates.  Assumes the first header of
% the series as input.
% Possible Types:
%  'RAS'       - Convert from image coordinates (ijk) to RAS
%  'XYZ'       - Convert from image coordinates (ijk) to Dicom (xyz)
%  'Rotate'    - Return rotation from ijk to xyz
%  'Scale'     - Return scale from ijk to xyz
%  'Translate' - Return translation from ijk to xyz
%  'Half'      - Return offset translation from ijk
%  'XYZToRAS'  - Return permutation matrix from xyz to ras
%
% The transforms are constructed as:
% RAS = XYZToRAS * Translate * Rotate * Scale * Half;
% XYZ = Translate * Rotate * Scale * Half;

if ~exist ( 'Type', 'var' )
  Type = 'RAS';
end
  
Header = Headers(1);

% Determine if we have DICOM or GE Signa
if ~isstruct ( Header )
  error ( 'Header is not a structure!' );
end

if ~isfield ( Header, 'mri' ) & ~isfield ( Header, 'PixelSpacing' ) & ~isfield ( Header, 'NumVols' )
  error ( 'Header must be a Signa, DICOM, or AVW header!' );
end

% For the two types of headers, need:
% RowOrientation in xyz coordinates, from upper left to lower left
% ColOrientation in xyz coordinates, from upper left to upper right
% Spacing in mm
% Origin in mm xyz coordinates

% Defaults, transverse
RowOrientation = [1 0 0];
ColOrientation = [0 1 0];
Spacing = [1 1 1];
Origin = [0 0 0];




if isfield ( Header, 'mri' )
  % Corners are [Upper left; Upper right; Lower left; Lower right;]
  % Reported in RAS coordinates
  
  PCoord = Header.mri.corners;
  
  % These need to be in xyz coordinates, not RAS as stored in the
  % GE Signa header...
  
  % Compute in RAS
  RowOrientation = PCoord(2,:) - PCoord(1,:);
  ColOrientation = PCoord(3,:) - PCoord(1,:);
  % Normalize and into XYZ
  RowOrientation = [-1 -1 1] .* RowOrientation ./ norm (RowOrientation);
  ColOrientation = [-1 -1 1] .* ColOrientation ./ norm (ColOrientation);

  Spacing = [Header.mri.pixsize Header.mri.slice_thickness+Header.mri.scanspacing];

  % XYZ is [-1 -1 1] .* RAS
  Origin = PCoord(1,:) .* [-1 -1 1];

end

if isfield ( Header, 'PixelSpacing' )
  % Now on to DICOM
  RowOrientation = Header.ImageOrientationPatient(1:3);
  ColOrientation = Header.ImageOrientationPatient(4:6);
    if isfield(Header, 'SpacingBetweenSlices')
    Spacing = [Header.PixelSpacing' Header.SpacingBetweenSlices];   
    else
      if length(Headers) > 1
        Z = sqrt ( sum ( ( Headers(2).ImagePositionPatient - Headers(1).ImagePositionPatient ).^2) );
        Spacing = [Header.PixelSpacing' Z];
      else    
     Transform=[]
      end
    end 
  Origin = Header.ImagePositionPatient;
end

if isfield ( Header, 'NumVols' )
    % AVW structure
    if isfield ( Header, 'RelativeImageOrientationPatient' )
        a = SplitField ( Header.RelativeImageOrientationPatient, '\' );
        RowOrientation = a(1:3);
        ColOrientation = a(4:6);
    elseif isfield ( Header, 'Orientation' )
        switch Header.Orientation
            case 'Transverse'
                % Don't need to do anything
            case 'Coronal'
                RowOrientation = [1 0 0];
                ColOrientation = [0 0 1];
            case 'Sagittal'
                RowOrientation = [0 1 0];
                ColOrientation = [0 0 1]
            
        end
    end
    if isfield ( Header, 'VoxelWidth' )
        Spacing = [Header.VoxelWidth Header.VoxelHeight, Header.VoxelDepth];
    end
end

% Form the Transformation Matrix
SliceOrientation = cross ( RowOrientation, ColOrientation );
Rotate = [ColOrientation(:)' 0; RowOrientation(:)' 0; SliceOrientation(:)' 0; 0 0 0 1]';
Scale = diag ( [Spacing(:)' 1], 0 );
Half = [1 0 0 -.5;
        0 1 0 -.5;
        0 0 1 -1;
        0 0 0 1];

Half = [1 0 0 -1;
        0 1 0 -1;
        0 0 1 -1;
        0 0 0 1];

Translate = eye ( 4 );
Translate(1:3,4) = Origin(:);
XYZToRAS = [-1 0 0 0;
            0 -1 0 0;
            0 0 1 0;
            0 0 0 1];

%  'RAS'       - Convert from image coordinates (ijk) to RAS
%  'XYZ'       - Convert from image coordinates (ijk) to Dicom (xyz)
%  'Rotation'  - Return rotation from ijk to xyz
%  'Scale'     - Return scale from ijk to xyz
%  'Translate' - Return translation from ijk to xyz
%  'Half'      - Return offset translation from ijk
%  'XYZToRAS'  - Return permutation matrix from xyz to ras

XYZ = Translate * Rotate * Scale * Half;
RAS = XYZToRAS * XYZ;
if isfield ( Header, 'NumVols' )
    if isfield ( Header, 'Orientation' )
        if (Header.Orientation=='niftiRAS')
%            RAS=diag([Spacing(:)',1],0);
             RAS=Header.niftiRASxform;
        end
    end
end
eval ( sprintf ( 'Transform = %s;', Type ) );

function a = SplitField ( Value, d )
  a = [];
  r = Value;
  while length ( r )
    [t, r] = strtok ( r, d );
    a = [a str2num(t)];
  end
  