function [NewImage, Results] = WarpPhantom ( Image, ImageData, Baseline, Calibration, varargin )
% WarpPhantom ( Image, ImageData, Baseline, Calibration )
%    Image - Phantom data
%    ImageData - Structure containing info about the phantom
%    Baseline - CalibrationData structure for baseline phantom data
%    Calibration - CalibrationData structure for calibration
%    phantom belonging to this phantom
% 
% Segment the original phantom
% Warp using the baseline and calibration data, and report

  % Defaults
  P = WarpDefaults;
  
  if nargin >= 5
    Parameters = varargin{1};
  else
    Parameters = P;
  end
  
  % Use the defaults, unless overridden
  if length ( Parameters )
    names = fieldnames ( P );
    for nn = 1:length(names)
      name = names{nn};
      if isfield ( Parameters, name )
        P = setfield ( P, name, getfield ( Parameters, name ) );
      end
    end
  end

  % Reorder the Calibration spheres
  Calibration = ReorderSpheres ( Baseline, Calibration );
  
  % Note: In GE Signa DICOM files, tag 0019,10fF is the Table delta
  
  % Want to do the Thin Plate Spline as a deformation of RAS space,
  % not Canonical space.  So, transform the Baseline Canonical COM
  % to RAS space of the Calibration phantom.  This becomes the TPS
  % transform.  We transform the Image based no the ImageData
  % transformations, warp using the TPS, and back to voxel space.

  % Voxel coordinate of BaselineCanCOM in output voxel coordinate
  % system must get it's value from voxel coordinate of same sphere
  % in original image, i.e. CalibrationCOM
  % Thus: CalibrationPointsCOM(1,:) == tforminv ( BaselinePointsCOM(1,:)', Warp )

  % Find the Calibration space TPS.  Start by putting the
  % BaselineCanonicalCOM into the RAS coordinate system of the
  % calibration scan.
  CalibrationPointsRAS = [Calibration.Spheres.CenterRAS];
  RASToCanonicalTForm = maketform ( 'affine', Calibration.RASToCanonicalTransform' );
  BaselinePointsRAS = tforminv ( [Baseline.Spheres.CanonicalCOM]', RASToCanonicalTForm )';
  
  
  % Let's do this slightly different.  Currently we directly
  % transform the Baseline spheres into the RAS of the Calibration
  % space.  Now let's try to align the spheres using the Orthogonal
  % Procrusties problem.
  OPP = OrthogonalProcrustes ( [Calibration.Spheres.CanonicalCOM], [Baseline.Spheres.CanonicalCOM] );
  
  CalibrationPointsCan = [Calibration.Spheres.CanonicalCOM]';
  
  % Take the OPP out for the moment
  % OPP = eye ( 4 );
    
  OPPTForm = maketform ( 'affine', OPP' );
  BaselinePointsCan = tformfwd ( [Baseline.Spheres.CanonicalCOM]', OPPTForm );
  BaselinePointsRAS = tforminv ( BaselinePointsCan, RASToCanonicalTForm )';
  
  D = CalibrationPointsCan - BaselinePointsCan;
  D = sqrt ( sum ( D.^2 ) );
  disp ( sprintf ( 'Canonical max distance: %f', max ( D ) ) ); 
  
  switch lower ( P.SplineType )
    case { 'tps', 'vs', 'ebs', 'ebrs' }
     % The spline must transform a BaselineRAS point into
     % CalibrationRAS space.
     % Note: we do things this way because the resampling code does
     % a tforminv.  TPS are (generally) not invertable, and so we
     % only provide the inverse transform, that is where is the
     % point in Baseline space, given a point in Calibration space. 
     W = TPS ( BaselinePointsRAS, CalibrationPointsRAS, ...
               P.SplineType, ...
               'Alpha', P.Alpha, ...
               'Stiffness', P.Stiffness );
    
     % Create the TPS TForm
     WarpCalibrationToBaselineTForm = maketform ( 'custom', 3, 3, [], @TPSValue, W );
   case { 'sphericalharmonics' }
    W = SphericalHarmonics ( CalibrationPointsRAS, BaselinePointsRAS );
     % Create the TPS TForm
     WarpCalibrationToBaselineTForm = maketform ( 'custom', 3, 3, [], @SphericalHarmonicsValue, W );
    case { 'affine' }
     % Create an affine transformation
     C = CalibrationPointsRAS;
     B = BaselinePointsRAS;
     C = [C; ones(1,size(C,2))];
     B = [B; ones(1,size(B,2))];
     T = B * C' * inv ( C * C' );
     T = T';
     T(:,4) = [0.0 0.0 0.0 1.0]';
     WarpCalibrationToBaselineTForm = maketform ( 'affine', T );
     
   case { 'poly' }
    % Create the parameters
    
   otherwise
    error ( sprintf ( 'WarpPhantom: Unknown spline type %s', P.SplineType ) );
  end
    
  % Create the Image transform
  VoxelToRASTForm = maketform ( 'affine', ImageData.VoxelToRASTransform' );
    
  
  % tforminv applies T from first to last,
  % i.e. if T = [T1; T2; T3], 
  % tforminv ( tforminv ( tforminv ( pt, T1 ), T2 ), T3 )
  % == tforminv ( pt, T )
  T = [ ...
      fliptform(VoxelToRASTForm); ...
      WarpCalibrationToBaselineTForm; ...
      VoxelToRASTForm; ...
      ];
  
  Warp = maketform ( 'composite', T );

  TranslateTForm = maketform ( 'affine', [1 0 0 0; 0 1 0 0; 0 0 1 -P.SliceNumber+1; 0 0 0 1]' );
  WarpSlice = maketform ( 'composite', [TranslateTForm; T] );
  
  R = makeresampler ( P.InterpolationMethod, P.PadMethod );

  % B = TFORMARRAY(A,T,R,   TDIMS_A,  TDIMS_B,  TSIZE_B,   TMAP_B,F)
  Image = squeeze ( Image );
  % disp ( 'Warping Image' );
  tic
  if P.SliceNumber == -1
    if P.DoWarp
      NewImage = tformarray ( Image, Warp, R, [1 2 3], [1 2 3], size(Image), [], 0 );
    else
      % Just copy over for speed
      % warning ( 'WarpPhantom: Not warping data!' );
      NewImage = Image;
    end
  else
    sz = size(Image);
    sz(3) = 1;
    if P.DoWarp
      NewImage = tformarray ( Image, WarpSlice, R, [1 2 3], [1 2 3], sz, [], 0 );
    else
      NewImage = Image(:,:,P.SliceNumber);
    end
  end
  % NewImage = abs ( NewImage );
  WarpTime = toc;
  if P.PrintWarpTime
    disp ( sprintf ( 'Transform time %0.2f minutes (%f seconds)', WarpTime / 60., WarpTime ) )
  end

  Results = struct([]);
  Results(1).WarpTime = WarpTime;
  Results.WarpParameters = P;
  Results.Warp = Warp;
  
  Results.WarpSlice = WarpSlice;

  Results.OriginalIsosurfaceVolume = -1;
  Results.WarpedOriginalIsosurfaceVolume = -1;
  Results.NewIsosurfaceVolume = -1;
  
  if Parameters.CalculateIsosurface
    % Calculate the before and after Isosurface volume
    T = OtsuThreshold ( Image );

    Surface = isosurface ( Image, T );
    WarpedSurface = Surface;
    WarpedSurface.vertices = tforminv ( Surface.vertices, Warp );
    Results.OriginalIsosurfaceVolume = VolumeFromIsosurface ( Surface );
    Results.WarpedOriginalIsosurfaceVolume = VolumeFromIsosurface ( WarpedSurface );

    if P.SliceNumber == -1
      T = OtsuThreshold ( NewImage );
      NewSurface = isosurface ( NewImage, T );
      Results.NewIsosurfaceVolume = VolumeFromIsosurface ( NewSurface );
    else
      Results.NewIsosurfaceVolume = -1;
    end    
  end
  
  
  if P.SliceNumber ~= -1
    Image = Image(:,:,P.SliceNumber);
  end

  if P.DoWarp & P.CalculateVolume
    % Otsu threshold
    T = OtsuThreshold ( Image );
    C = length ( find ( Image(:) >= T ) );
    Results.OriginalOtsuThreshold = T;
    Results.OriginalOtsuCount = C;
    % disp ( sprintf ( 'Original Image Otsu Volume: %d', C ) );

    % Otsu on Warped data
    % Momentarily, use the threshold from the original
    % T = OtsuThreshold ( NewImage );
    C = length ( find ( NewImage(:) >= T ) );
    Results.WarpedOtsuThreshold = T;
    Results.WarpedOtsuCount = C;
    % disp ( sprintf ( 'Warped Image Otsu Volume: %d', C ) );

    [P,A] = VolumeBySlice ( Image );
    Results.OriginalImageContourPixels = P;
    Results.OriginalImageContourArea = A;
    % disp ( sprintf ( 'Original Image Contour: %d pixels %f area', P, A ) );
  
    [P,A] = VolumeBySlice ( NewImage );
    Results.WarpedImageContourPixels = P;
    Results.WarpedImageContourArea = A;
    % disp ( sprintf ( 'Warped Image Contour: %d pixels %f area',
    % P, A ) );
  end
  
  
  % disp ( sprintf ( 'Original isosurface: %.2f', Results.OriginalIsosurfaceVolume ) );
  % disp ( sprintf ( 'Warped Original isosurface: %.2f', Results.WarpedOriginalIsosurfaceVolume ) );
  % disp ( sprintf ( 'New isosurface: %.2f', Results.NewIsosurfaceVolume ) );
  

  

